#include "StdAfx.h"
#include ".\inifile.h"
#include "util.h"

#ifndef ASSERT
#define ASSERT _ASSERT
#define AfxIsValidString(x) TRUE
#endif

CIniFile::CIniFile(void)
{
}

CIniFile::~CIniFile(void)
{
}

//*******************************************************************
// 
//*******************************************************************

void CIniFile::SetIniFileName(LPCTSTR filename)
{
	ASSERT( AfxIsValidString(filename) );
	m_iniFilePath = filename;
	::CreateDirAll(m_iniFilePath.c_str());
}

void CIniFile::SetIniFileNameDefault()
{
	TCHAR fname[MAX_PATH];
	TCHAR *pos;

	::GetModuleFileName(NULL, fname, MAX_PATH);
	ASSERT( AfxIsValidString(fname) );

	pos = _tcsrchr(fname, _T('.'));
	if( pos != NULL ){
		*pos = _T('\0');
	}
	_tcscat_s(fname, _T(".ini"));

	SetIniFileName(fname);
}


//*******************************************************************
// 
//*******************************************************************

bool CIniFile::ReadBool(LPCTSTR section, LPCTSTR entry, bool bDefault)
{
	return ::GetPrivateProfileInt(section, entry, bDefault,
					m_iniFilePath.c_str()) != 0;
}

int CIniFile::ReadInt(LPCTSTR section, LPCTSTR entry, int nDefault)
{
	return ::GetPrivateProfileInt(section, entry, nDefault,
					m_iniFilePath.c_str());
}


void CIniFile::ReadString(tstring &out, LPCTSTR section, LPCTSTR entry, LPCTSTR defaultString)
{
	TCHAR *buf;
	DWORD ret, size;

	size = 0;
	for(;;){
		size += INI_BUF_SIZE;
		buf = new TCHAR[size];
		ret = ::GetPrivateProfileString(section, entry,
					defaultString, buf, size, m_iniFilePath.c_str());
		if( ret != size - 1 ) break;
		delete[] buf;
	}
	UnescapeIniStr(buf, out);
	delete[] buf;
}

void CIniFile::ReadSection(LPCTSTR section, EntryList &list)
{
	tstring str;
	TCHAR *buf;
	DWORD ret, size;
	LPTSTR key, val;

	size = 0;
	for(;;){
		size += INI_BUF_SIZE;
		buf = new TCHAR[size];
		ret = ::GetPrivateProfileSection(section, buf, size, m_iniFilePath.c_str());
		if( ret != size - 2 ) break;
		delete[] buf;
	}

	key = buf;
	while( *key != _T('\0') ){
		val = _tcschr(key, _T('='));
		if( val == NULL ) break;
		*val++ = '\0';

		UnescapeIniStr(val, str);
		list[key] = str;

		key = val + _tcslen(val) + 1;
	}
	delete[] buf;
}

void CIniFile::GetSectionList(SectionList &list)
{
	TCHAR *buf;
	DWORD ret, size;
	LPTSTR section;

	size = 0;
	for(;;){
		size += INI_BUF_SIZE;
		buf = new TCHAR[size];
		ret = ::GetPrivateProfileSectionNames(buf, size, m_iniFilePath.c_str());
		if( ret != size - 2 ) break;
		delete[] buf;
	}

	section = buf;
	while( *section != _T('\0') ){
		list.push_back(section);
		section += _tcslen(section) + 1;
	}
	delete[] buf;
}

//*******************************************************************
// 
//*******************************************************************

void CIniFile::Write(LPCTSTR section, LPCTSTR entry, int value)
{
	char buf[16];

	_itoa_s(value, buf, 10);
	::WritePrivateProfileString(section, entry, buf, m_iniFilePath.c_str());
}

void CIniFile::Write(LPCTSTR section, LPCTSTR entry, LPCTSTR str)
{
	tstring escaped;

	EscapeIniStr(str, escaped);
	::WritePrivateProfileString(section, entry, escaped.c_str(), m_iniFilePath.c_str());
}

//*******************************************************************
// 
//*******************************************************************
void CIniFile::EscapeIniStr(LPCTSTR in, tstring &out)
{
	_ASSERT( in != NULL );
	LPCTSTR pos;

	out = "";
	out.reserve(lstrlen(in)+1);
	for( pos = in; *in != _T('\0'); in++ ){
		switch( *in ){
		case _T('\t'):
			out += _T("\\t");
			break;
		case _T('\r'):
			out += _T("\\r");
			break;
		case _T('\n'):
			out += _T("\\n");
			break;
		case _T('\\'):
			out += _T("\\\\");
			break;
		default:
			out += *in;
		}
	}
}

void CIniFile::UnescapeIniStr(LPCTSTR in, tstring &out)
{
	_ASSERT( in != NULL );
	LPCTSTR pos;
	bool bEscape = false;

	out = "";
	out.reserve(lstrlen(in)+1);
	for( pos = in; *in != _T('\0'); in++ ){
		if( bEscape ){
			switch( *in ){
			case _T('t'):
				out += _T('\t');
				break;
			case _T('r'):
				out += _T('\r');
				break;
			case _T('n'):
				out += _T('\n');
				break;
			default:
				out += *in;
				break;
			}
			bEscape = false;
		}else{
			if( *in == _T('\\') ){
				bEscape = true;
			}else{
				out += *in;
			}
		}
	}
}

