#include "stdafx.h"
#include "cs_conv.h"
#include "DynamicArray.h"

int ToUnicode(UINT from, LPCSTR in, std::wstring &out)
{
	int unicodeLen;

	unicodeLen = ::MultiByteToWideChar(from, 0, in, -1, NULL, 0);
	if( unicodeLen == 0 ) return FALSE;

	DynamicArray<wchar_t> wbuf(unicodeLen);
	::MultiByteToWideChar(from, 0, in, -1, wbuf, unicodeLen);

	out = wbuf;
	return TRUE;
}

int UnicodeTo(UINT to, LPCWSTR in, std::string &out)
{
	int sjisLen;

	sjisLen = ::WideCharToMultiByte(to, 0, in, -1, NULL, 0, NULL, NULL);
	if( sjisLen == 0 ) return FALSE;

	DynamicArray<char> sbuf(sjisLen);
	::WideCharToMultiByte(to, 0, in, -1, sbuf, sjisLen, NULL, NULL);

	out = sbuf;
	return TRUE;
}

// st  ed ܂ł̕JISR[hǂf
bool IsJIS(LPCSTR st, LPCSTR ed)
{
	bool bJis = false;
	for( ; st < ed; st++ ){
		if( *st == '\x1b' ){ bJis = true; }
		else if( *st > 0x7F ){ return true; }
	}
	return bJis;
}

// in_st  in_ed ܂łSJISJISɕϊ
// out ̃obt@Ɋi[B
// out ɂ͏\ȃobt@̂ƂB
// ϊ㕶̍Ō̃kւ̃|C^Ԃ
enum JIS_STATE{
	JE_ASCII,
    JE_KANJI,
	JE_KANA,
};

LPSTR JIStoSJIS(LPSTR out, LPCSTR in_st, LPCSTR in_ed)
{
	JIS_STATE state = JE_ASCII;

	for( LPCSTR pos = in_st; pos < in_ed; pos++ ){
		if( *pos == 0x1b )
		// ؂ւV[PX
		{
			if( (pos[1] == '$') && ((pos[2] == 'B') || (pos[2] == '@'))){
				state = JE_KANJI;
			}else if( (pos[1] == '(') && ((pos[2] == 'B') || (pos[2] == 'J'))){
				state = JE_ASCII;
			}else if( (pos[1] == '(') && (pos[2] == 'I')){
				state = JE_KANA;
			}else{
				// FłȂV[PX
				_ASSERT(FALSE);
			}
			pos += 2;
		}
		else
		// ʏ핶
		{
			if( state == JE_KANJI )
			// 2 byte
			{
				if( *pos == 0x0E ){
					*out++ = *pos;
				}else if( *pos == 0x0F ){
					*out++ = *pos;
				}else if( *pos & 1 ){
					if( !(pos + 1 < in_ed) ){
						_ASSERT(FALSE);
						break;
					}
					*out++ = (*pos >> 1) + ((*pos < 0x5F) ? 0x71: 0xB1);
					pos++;
					*out++ = *pos + ((*pos < 0x60) ? 0x1F: 0x20);
				}else{
					if( !(pos + 1 < in_ed) ){
						_ASSERT(FALSE);
						break;
					}
					*out++ = (*pos >> 1) + ((*pos < 0x5F) ? 0x70: 0xB0);
					pos++;
					*out++ = *pos + 0x7E;
				}
			}
			else if( state == JE_KANA )
			// pJi
			{
				*out++ = (*pos | 0x80);
			}
			else
			// ̑
			{
				*out++ = *pos;
			}
		}
	}
	*out = '\0';
	return out;
}

void SJIStoJIS(LPCSTR sjis, std::string &jis)
{
	JIS_STATE state = JE_ASCII;
	char c;

	for( LPCSTR pos = sjis; *pos; pos++ ){
		c = *pos;
		if( c < 0x80 ){
			if( state != JE_ASCII ){
				state = JE_ASCII;
				jis += "\x1B(B";
			}
			jis += c;
		}else if( 0xA0 <= c && c <= 0xDF ){
			if( state != JE_KANA ){
				state = JE_KANA;
				jis += "\x1B(I";
			}
			jis += (c & 0x7F);
		}else{
			if( state != JE_KANJI ){
				state = JE_KANJI;
				jis += "\x1B$B";
			}
			pos++;
			if( *pos == '\0' ) break;

			if( *pos < 0x9F ){
				jis += (c << 1) + ((c < 0x7F) ? 0x1F: -0x61);
				jis += *pos - ((*pos > 0x7E) ? 0x20: 0x1F);
			}else{
				jis += (c << 1) + ((c < 0x7F) ? 0x20: -0x60);
				jis += *pos - 0x7E;
			}
		}
	}
	if( state != JE_ASCII )
		jis += "\x1B(B";
}

int JisStrLenFromSjis(LPCSTR sjis)
{
	JIS_STATE state = JE_ASCII;
	int len = 0;
	char c;

	for( LPCSTR pos = sjis; *pos; pos++ ){
		c = *pos;
		if( c < 0x80 ){
			if( state != JE_ASCII ){
				state = JE_ASCII;
				len += 3;
			}
			len++;
		}else if( 0xA0 <= c && c <= 0xDF ){
			if( state != JE_KANA ){
				state = JE_KANA;
				len += 3;
			}
			len++;
		}else{
			if( state != JE_KANJI ){
				state = JE_KANJI;
				len += 3;
			}
			pos++;
			if( *pos == '\0' ) break;

			len += 2;
		}
	}
	if( state != JE_ASCII )
		len += 3;

	return len;
}

